Attribute VB_Name = "mCore"
Option Explicit

'Types
    Public Type tPoint
        x As Long
        y As Long
    End Type

'Declares
    Public Declare Function GetTickCount Lib "kernel32" () As Long
    
    Public Declare Function MoveToEx Lib "gdi32" (ByVal iDC As Long, ByVal x As Long, ByVal y As Long, iPoint As tPoint) As Long
    Public Declare Function LineTo Lib "gdi32" (ByVal iDC As Long, ByVal x As Long, ByVal y As Long) As Long
    
    Public Declare Function DeleteObject Lib "gdi32" (ByVal iObject As Long) As Long
    Public Declare Function SelectObject Lib "gdi32" (ByVal iDC As Long, ByVal iObject As Long) As Long
    
    Public Declare Function CreateSolidBrush Lib "gdi32" (ByVal iColor As Long) As Long
    Public Declare Function CreatePen Lib "gdi32" (ByVal iStyle As Long, ByVal iWidth As Long, ByVal iColor As Long) As Long

'Data type definition
    Public Type tStar
        x As Single
        y As Single
        z As Single
    End Type

'The stars array
    Public StarCount As Long
    Public Star() As tStar

'Screen size
    Public ScreenW As Long
    Public ScreenH As Long
    
    Public OffsetX As Long
    Public OffsetY As Long

'Setup
    Public Speed As Single
    
    Public SpreadX As Single
    Public SpreadY As Single
    Public SpreadZ As Single

Public Function DrawLine(DC As Long, x1 As Long, y1 As Long, x2 As Long, y2 As Long, iColor As Long)
    Dim Pen As Long
    Dim Brush As Long
    
    Dim OldPen As Long
    Dim OldBrush As Long
    
    Dim TempPos As tPoint
    
    'Create pen
    Pen = CreatePen(0, 1, iColor)
    OldPen = SelectObject(DC, Pen)
    
    'Create brush
    Brush = CreateSolidBrush(iColor)
    OldBrush = SelectObject(DC, Brush)
    
    'Draw line
    MoveToEx DC, x1, y1, TempPos
    LineTo DC, x2, y2
    
    'Restore brush
    SelectObject DC, OldBrush
    DeleteObject Brush
    
    'Restore pen
    SelectObject DC, OldPen
    DeleteObject Pen
End Function


Public Sub DrawStars(DC As Long)
    Dim A As Long
    
    Dim FromPos As tPoint
    Dim ToPos As tPoint
    
    Dim Brightness As Single
    
    For A = 0 To StarCount
        With Star(A)
            'Get start point
            FromPos.x = (.x / .z) + OffsetX
            FromPos.y = (.y / .z) + OffsetY
            
            'Get end point
            ToPos.x = (.x / (.z + Speed)) + OffsetX
            ToPos.y = (.y / (.z + Speed)) + OffsetY
            
            'Get brightness from distance (here ^ 2 smooths the fading of darker stars)
            Brightness = (1 - (.z / SpreadZ)) ^ 2 * 255
            
            'Draw the line
            DrawLine DC, FromPos.x, FromPos.y, ToPos.x, ToPos.y, RGB(Brightness * 1, Brightness * 0.8, Brightness * 0.4)
        End With
    Next
End Sub

Public Sub MoveStars()
    Dim A As Long
    
    For A = 0 To StarCount
        With Star(A)
            'Move towards screen
            .z = .z - Speed
            
            If .z < 0.1 Then
                'Reset star when in front of screen
                .x = (Rnd - 0.5) * SpreadX
                .y = (Rnd - 0.5) * SpreadY
                .z = SpreadZ
            End If
        End With
    Next
End Sub


Public Sub SetupStars(iCount As Long)
    Dim A As Long
    
    'Create stars array
    StarCount = iCount
    ReDim Star(StarCount)
    
    'Set random positions
    For A = 0 To StarCount
        With Star(A)
            .x = (Rnd - 0.5) * SpreadX
            .y = (Rnd - 0.5) * SpreadY
            .z = Rnd * SpreadZ
        End With
    Next
End Sub


